/**********************************************************************
 *    amlb-calc.p
 *    (mat_calc.p)
 *    Augustus Mickus, 2003-06-24
 *		email: augustus@email.lt
 **********************************************************************
 *    calculator
 *    solves simple mathematical expression
 *    remarks
 *    - available signs: "+", "-", "*", "/". brackets are not avail.;
 *		- unary minus is alowed;
 *    - sequence is "normal" (division, multip., substraction, addition)
 *		- numbers must be w/o thousand separators. decim. symbol: "."
 *    - no other simbols are allovew
 *		params:
 *		- in  sExpr - expression (e.g. "-44+5-4--8")
 * 	- out dVal  - result. (? in case of error)
 *		return-value
 *		- in case of err, there will be returned description of err 
 *			folowed by symbols "2," in return-val 
 *			(e.g.: "2,Error description")
*/
&scop sep chr(1)

/*********************************************************************/
def input param sExpr as char no-undo.
def output param dVal as dec no-undo init ?.
/*********************************************************************/

sExpr = replace(sExpr, " ", "").
run calc (sExpr, output dVal).

return return-value.
/*********************************************************************/


function isNum log (ch as char):
   if ch = "" then return false.
   return (index("012345678.", ch)>0).
end function.

function isOpr log (ch as char):
   return (index("*+-/", ch)>0).
end function.

/* function isStr log (ch as char):
   return (index("ABCDEFGHIJKLMNOPQRSTUVWXYZ_", ch) > 0).
end function. */

/* parse and split string */
function str2lst char (sStr as char):
   def var ls     as char no-undo init "".
   def var ch     as char no-undo init "".
   def var chPrv  as char no-undo init "".
   def var i      as int  no-undo.
   def var iLen   as int  no-undo.
   iLen = length(sStr).
   do i = 1 to iLen:
      ch = substr(sStr, i, 1).
      if isNum(ch) and not isNum(chPrv)
/*            or isStr(ch) and not isStr(chPrv) */
            or isOpr(ch)
         then ls = ls + {&sep}.
      ls = ls + ch.
      chPrv = ch.
   end.
   return trim(ls, {&sep}).
end function.

def temp-table tEqa no-undo
   field pos   as int  init 0
   field opr   as char form "x(1)" init ""
   field neg   as log  init no
   field val   as dec  init 0
   .

def buffer bfTmp for tEqa.

function getPrevPos int (iPos as int):
   for each bfTmp where bfTmp.pos < iPos by bfTmp.pos desc:
      return bfTmp.pos.
   end.
   return 0.
end function.

/* suskaiciuoja visas vieno tipo operacijas (dalybas, daugybas ir pan) */
procedure calc-opr:
   def input param sOpr as char no-undo.
   def input param sPrc as char no-undo.
   def var iPrv   as int  no-undo init 0.
   def buffer BF-1 for tEqa.
   def buffer BF-2 for tEqa.
   repeat:
      find first BF-2 where BF-2.opr = sOpr no-error.
      if not avail BF-2 then leave.
      iPrv = getPrevPos(BF-2.pos). if iPrv = 0 then return "2,Incorrect syntax".
      find first BF-1 where BF-1.pos = iPrv.
      run value("clc-" + sPrc) (BF-1.val, BF-2.val, output BF-1.val) no-error.
         if error-status:error then return "2,Internal error".
         if return-value begins "2" then return return-value.
      delete BF-2.
   end.
   return.
end procedure.

/* pagrindiniai skaiciavimai */
procedure calc:
   def input  param sEqa as char no-undo.
   def output param dRes as dec  no-undo init ?.
   def var lsEqa  as char no-undo init "".
   def var sVal   as char no-undo init "".
   def var sOpr   as char no-undo init "".
   def var iLen   as int  no-undo init 0.
   def var i      as int  no-undo init 0.

   /*** uzpildom tEqa
   */
   for each tEqa: delete tEqa. end.
   lsEqa = {&sep} + str2lst(sEqa).
   iLen  = num-entries(lsEqa, {&sep}).
   i = 1.
   repeat while i < iLen:
      create tEqa.
      tEqa.pos = i.
      sOpr = entry(i, lsEqa, {&sep}). i = i + 1.
      sVal = entry(i, lsEqa, {&sep}). i = i + 1.
      if sVal = "-" then do: /* unarinis minusas*/
         tEqa.neg = yes.
         sVal = entry(i, lsEqa, {&sep}). i = i + 1.
      end.
      tEqa.opr = sOpr.
      tEqa.val = dec(sVal) no-error.
      if error-status:error then return "2,Unrecognised operand: " + sVal.
   end.

   /*** skaiciuojam
   */

   /* neigiami skaiciai */
   for each tEqa where tEqa.neg:
      tEqa.val = - tEqa.val.
   end.

   /* dalyba */
   run calc-opr ("/", "div"). if return-val begins "2" then return return-val.
   run calc-opr ("*", "mul"). if return-val begins "2" then return return-val.
   run calc-opr ("-", "sub"). if return-val begins "2" then return return-val.
   run calc-opr ("+", "add"). if return-val begins "2" then return return-val.

   find first tEqa.
   dRes = tEqa.val.
   return .
end procedure.


/******** clc operatoriai ********/
procedure clc-div:
   def input  param d1   as dec no-undo.
   def input  param d2   as dec no-undo.
   def output param dOut as dec no-undo init ?.
   if d2 = 0 then return "2,Division by zero is imposible".
   dOut = d1 / d2.
   return.
end procedure.
procedure clc-mul:
   def input  param d1   as dec no-undo.
   def input  param d2   as dec no-undo.
   def output param dOut as dec no-undo init ?.
   dOut = d1 * d2.
   return.
end procedure.
procedure clc-add:
   def input  param d1   as dec no-undo.
   def input  param d2   as dec no-undo.
   def output param dOut as dec no-undo init ?.
   dOut = d1 + d2.
   return.
end procedure.
procedure clc-sub:
   def input  param d1   as dec no-undo.
   def input  param d2   as dec no-undo.
   def output param dOut as dec no-undo init ?.
   dOut = d1 - d2.
   return.
end procedure.


/*
test:
def var d as dec.
run calc.p ("-44+5-4--8", output d).
display d.
message return-val.
*/
