/**********************************************************************
 *		amlb-equa.p
 * 	(mat_equation.p)
 *    Augustus Mickus, 2003-06-24
 *		email: augustus@email.lt
 **********************************************************************
 *		checks, is equation/inequation correct
 *		params:
 *		- in sCondit - equations to be checked. 
 *			there may be few equations joined with logical OR (||)
 *    	e.g.: "44 = 33 + 11 || 45 > 55 || 12 >= -12"
 *		- out bOk - result
 *		return-value on error will return error description followed by
 *			symbols "2,"
*/


/*********************************************************************/
def input  param sCondit as char no-undo.
def output param bOk     as log  no-undo init no.
/*********************************************************************/
function str_findExistItem char (sStr as char, lsItm as char, sSep as char) forward.

def var sEqa   as char no-undo init "".
def var sSide1 as char no-undo init "".
def var sSide2 as char no-undo init "".
def var sEqop  as char no-undo init "".
def var i      as int  no-undo.

sCondit = replace(sCondit, "||", chr(1)).
do i = 1 to num-entries(sCondit, chr(1)):
   sEqa = entry(i, sCondit, chr(1)).
   sEqop = str_findExistItem(sEqa, ">=,<=,<>,=,<,>", ",").
   if sEqop = "" then return "2,Missing one of operators: >=, <=, <>, =, <, >".
	sEqa = replace(sEqa, sEqop, chr(1)).
   sSide1 = entry(1, sEqa, chr(1)).
   sSide2 = entry(2, sEqa, chr(1)).
   run istrue(sSide1, sSide2, sEqop, output bOk).
      if return-val begins "2" then return return-value.
   if bOk = yes then return.
end.
return.
/*********************************************************************/

procedure istrue:
   def input param sSide1 as char no-undo.
   def input param sSide2 as char no-undo.
   def input param sEqop  as char no-undo.
   def output param bTrue as log  no-undo init no.

   def var dVal1 as char no-undo init 0.
   def var dVal2 as char no-undo init 0.

   run amlb-calc.p (sSide1, output dVal1).
      if return-val begins "2" then return return-value.
   run amlb-calc.p (sSide2, output dVal2).
      if return-val begins "2" then return return-value.

   case sEqop:
   when "="  then bTrue = (dVal1 = dVal2).
   when "<"  then bTrue = (dVal1 < dVal2).
   when ">"  then bTrue = (dVal1 > dVal2).
   when "<=" then bTrue = (dVal1 <= dVal2).
   when ">=" then bTrue = (dVal1 >= dVal2).
   when "<>" then bTrue = (dVal1 <> dVal2).
   otherwise return "2,Incorrect syntax".
   end case.

   return "".
end procedure.
/*********************************************************************/

/* grazina pirma elementa is lsItm saraso, kuris egzistuoja sStr'e */
function str_findExistItem char (sStr as char, lsItm as char, sSep as char):
   def var i    as int  no-undo.
   def var sItm as char no-undo init "".
   do i = 1 to num-entries(lsItm, sSep):
      sItm = entry(i, lsItm, sSep).
      if index(sStr, sItm) > 0 then return sItm.
   end.
   return "".
end function.


